import urllib2
import json
import os
import shelve
import cPickle as pickle


def shelve_memoized(file_name):
	cache = shelve.open(file_name)
	def deco(f):
		def g(*args):
			if len(args) == 0 or args[0] is None:
				return f(*args)
			args_str = pickle.dumps(tuple(args))			
			try:
				return cache[args_str]
			except:
				result = f(*args)
				cache[args_str] = result
				cache.sync()
				return result
		return g
	return deco


@shelve_memoized("xkcd_info.db")
def get_info(comic_number=None):
	if comic_number is not None:
		print "Getting info for %s" % comic_number
	url = "http://xkcd.com/%sinfo.0.json" % ("" if comic_number is None 
	                                         else "%i/" % comic_number)
	return json.load(urllib2.urlopen(url))


def get_comic_numbers():
	latest = get_info()["num"]
	return [x for x in range(1, latest+1) if x != 404]


html_escape_table = {
    "&": "&amp;",
    '"': "&quot;",
    "'": "&apos;",
    ">": "&gt;",
    "<": "&lt;",
    }


def html_escape(text):
    """Produce entities within text."""
    return "".join(html_escape_table.get(c,c) for c in text)


def html_template(file_name, title, body, extra_head=""):
	html = """
<html>
  <head>
    <title>%s</title>
    %s
  </head>
  <body>
    %s
  </body>
</html>
""" % (title, extra_head, body)

	open(file_name, "w").write(html.encode('utf-8'))
	

def download(info):
	if not os.path.exists("images"):
		os.mkdir("images")

	if not os.path.exists("pages"):
		os.mkdir("pages")

	write_loc = os.path.join("images", info["img"].split('/')[-1])

	if not os.path.exists(write_loc):
		print "downloading %s..." % info["num"] 
		data = urllib2.urlopen(info["img"]).read()
		open(write_loc, "w").write(data)
	
	html = "<img src='%s' title='%s' alt='%s' />" % ("../" + write_loc,
	                                                 html_escape(info["alt"]),
	                                                 html_escape(info["title"]))

	html += "<dl>"
	for key in sorted(info.keys()):
		html += "<dt>%s</dt><dd>%s</dd>" % (key, html_escape(unicode(info[key])))
	html += "</dl>"

	html_template(os.path.join("pages", "%s.html" % str(info["num"])),
	              "xkcd %s: %s" % (info["num"], info["title"]),
	              html,
	              "<style type='text/css'>dt{font-weight: bold;}</style>")


def make_index(comic_numbers):
	html = "<h1>xkcd</h1><ul>"
	for num in comic_numbers:
		info = get_info(num)
		html += "<li>%s</li>" % (
			"<a href='%s'>%s</a>" % ("pages/%s.html" % num,
			                         "%s - %s" % (num,
			                                      info["safe_title"])))
	html += "</ul>"
	html_template("index.html", "xkcd", html)


def download_numbers(comic_numbers):
	make_index(comic_numbers)
	for info in map(get_info, comic_numbers):
		download(info)


if __name__ == "__main__":
	download_numbers(get_comic_numbers())
